// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package document

import (
	"fmt"
	"gitlab.schukai.com/oss/utilities/documentation-manager/environment"
	"io/ioutil"
	"os"
	"os/exec"
)

func runInlinePandoc(source string) string {

	tmpSource, err := ioutil.TempFile(os.TempDir(), "inline-source")
	if err != nil {
		environment.ExitWithError(2, "A temporary file cannot be created", err.Error())
	}

	defer os.Remove(tmpSource.Name())

	tmpResult, err := ioutil.TempFile(os.TempDir(), "inline-result")
	if err != nil {
		environment.ExitWithError(2, "A temporary file cannot be created", err.Error())
	}

	defer os.Remove(tmpResult.Name())

	tmpSource.WriteString(source)

	arguments := "/bin/env pandoc "

	arguments += "--variable fontsize=12pt "
	arguments += "--variable version=2.0 "
	arguments += tmpSource.Name() + " "
	arguments += "--from markdown "
	arguments += "--to latex "
	arguments += "--output=" + tmpResult.Name() + " "

	cmd := exec.Command("bash", "-c", arguments)
	_, err = cmd.Output()

	if err != nil {

		exitErr, ok := err.(*exec.ExitError)
		if ok {
			environment.ExitWithError(2, string(exitErr.Stderr))

		} else {
			environment.ExitWithError(2, err.Error())
		}

	}

	content, err := os.ReadFile(tmpResult.Name())
	checkError(err)

	return string(content)

}

func runPandoc(source string, outputName string, latexPath string, luaFilter string, verbose bool) error {

	arguments := "/bin/env pandoc "

	arguments += "--number-sections "
	arguments += "--variable \"geometry:a4paper,margin=2cm\" "
	arguments += "--variable fontsize=12pt "
	arguments += "--variable version=2.0 "
	arguments += source + " "
	arguments += "--pdf-engine=xelatex "
	arguments += "--from markdown "
	arguments += "--to pdf "
	if verbose {
		arguments += "--verbose "
	}
	arguments += "--listings "

	if latexPath != "" {
		arguments += "--template=" + latexPath + " "
	}

	if luaFilter != "" {
		arguments += "--lua-filter=" + luaFilter + " "
	}

	arguments += "--columns=5 "
	arguments += "--highlight-style espresso "
	arguments += "--toc "
	arguments += "--output=" + outputName + " "

	if verbose {
		fmt.Println(arguments)
	}

	cmd := exec.Command("bash", "-c", arguments)
	log, err := cmd.Output()

	if err != nil {

		if verbose {
			fmt.Println(string(log))
		}

		exitErr, ok := err.(*exec.ExitError)
		if ok {
			environment.ExitWithError(2, string(exitErr.Stderr))

		} else {
			environment.ExitWithError(2, err.Error())
		}

	}

	return err

}
