// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package environment

import "github.com/jessevdk/go-flags"

type HandlerStruct struct {
	Shutdown []*Hook
	Executor map[string]Handler
}

type Hook func()

type Handler struct {
	Init    func(*flags.Command)
	Execute func(*flags.Command)
}

var Handlers HandlerStruct

func init() {
	Handlers.Executor = make(map[string]Handler)
	Handlers.Shutdown = make([]*Hook, 0)
}

func (e *HandlerStruct) RemoveShutdownHook(h *Hook) *HandlerStruct {
	for i, f := range e.Shutdown {
		if f == h {
			e.Shutdown = append(e.Shutdown[:i], e.Shutdown[i+1:]...)
			return e
		}
	}
	return e
}

func (e *HandlerStruct) AddShutdownHook(h *Hook) *HandlerStruct {
	e.Shutdown = append(e.Shutdown, h)
	return e
}
